#include "../main.f"
c        **************************************************************
c        *
c        * ephook:  collection of subroutines which should be managed by
c        *      the user. 
c        **************************************************************
c         
c
c   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c   +  Note that besides the subroutines listed below in this module,
c   +  the following inquirey subroutines are available for use:
c   +
c   +  call epqFirstI(pos);
c         #include "ZepPos.h"
c         record /epPos/ pos ; output.( pos.x, pos.y, pos.z )
c                              First interaction position. If
c
c   +  call epqmat(i, mat): i; integer input.  component number
c   +                       mat: characeter*8 output.  mediea
c   +                            such as 'Si' of  the i-th component.
c   +  call epqstn(n):  n; integer output.  get current stack depth
c
c   +  call epqsTrack(n, sTrack): inquire stacked track.
c                    n; integer input. stack depth
c                   record /epTrack/ sTrack. Energy=0 means n is invalid
c   +  call epqevn(nev):  nev; integer output. Event number already created
c                              in this run.
c   +  call epqinc(aTrack)
c                 inquire incident particle.
c   +         record /epTrack/ aTrack
c   +             if multiple particles are incident, the first one is obtained
c
c   +  call epqncp(ncomp) inquire the total number of components
c                   integer ncomp  output.
c
c   +  call sqtevn(nev): nev; integer output.  total number of events
c   +                                           created so far.
c   +
c   +  call sqcont(icon):  icon; integer output. 
c   +                            icon == 0 ==>  first job
c   +                            icon != 0 ==>  continued job
c   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c        
c
c       *****************************
c       *        At this momemet, all the system level initialization 
c       *   has been ended.  Do your own initialization for all the
c       *   events.
c
        subroutine uiaev
        implicit none
#include  "Zevhnp.h"
#include  "Zptcl.h"
#include  "Zcode.h"
        include 'Zother.h'
        character*150 msg
c
        write(*,*) '------------------------------------------------'
        open(11, file='./other')
        read(11,*) MulLow, energy, xin, yin, zin, outatint
        close(11)
        writse(0, *) ' MulLow=',MulLow, ' energy=',energy,
     *      ' xin, yin,zin=',xin, yin, zin, ' outatint=',
     *        outatint
c        call cerrorMsg(msg, 1)
        if(outatint) then
           write(*,*)
     *     '#  "eta"  "y" "pt" " 1/pt" "E/E0(lab)" " code"'
        else
           write(*,'(a100)')
     *     '#  "eta" "y"  "pt" "1/pt"  "E/E0" "Eg" "E/E0(lab)"'
     *    //' " log(Eg)"  "cn"  "code" "tag" "x" "y"'
        endif
c
        tg.fm.p(1) = 0.
        tg.fm.p(2) = 0.

        tg.fm.p(4) = energy
c             make proton as colliding beam
        call cmkptc(knuc, regptcl, 1,   pj)
        call cmkptc(knuc, regptcl, 1,   tg)
        tg.fm.p(3) = -sqrt(energy**2 - tg.mass**2)
        pj.fm.p(1) =0.
        pj.fm.p(2) =0.
        pj.fm.p(4) = energy
        pj.fm.p(3) = sqrt(pj.fm.p(4)**2-pj.mass**2)
c             get equivalent lab energy; show it as info.
        call cbst1(1, tg, pj, plab)
        write(msg, *) ' equiv. lab E=', plab.fm.p(4)
        call cerrorMsg(msg, 1)
        end

c       **************************************************
c       *  At this moment, all the system level initialization for
c       *  an event has been ended.  Do your own initializaiton for
c       *  the event.   Note, at this moment, no incident particle
c       *  has been made yet.
c       *
        subroutine ui1ev
        implicit none
        integer  klena
        character*8 uid
        character*64  file
c        integer neve, getir(2)
c         open file for putting trace info. 
c         In your normal job, you should not take tarce info,
c          otherwise, you need lot of disk space and computation time.
c        *************************8    
        call cgetLoginN(uid)  ! cosmos function
        file = '/tmp/' // uid(1:klena(uid))// '/trace1'
        open(21, file=file)
c        *********************
 
        end
c       **************************************************
c       *  If the default incident particle treatment does not
c       *  suffice you, make your own incident particle here.
c       * (you can  make even multiple particles as  the incident).
c       *  If default is ok, don't touch this.
        subroutine usetip(icon)
        implicit none
        integer icon
#include "ZepTrack.h"
        include 'Zother.h'
        integer maxn
        parameter (maxn = 5000 )
        record /ptcl/ w(maxn), po
        record /epTrack/ aTrack

        integer ntp, i, currentev
        real*8 p, pt, pti, y, eta

        call ephookPPCol(w, ntp)


        call epqevn(currentev)

        do i = 1, ntp
           pt= sqrt(w(i).fm.p(1)**2 + w(i).fm.p(2)**2 )
           p = sqrt(pt**2 + w(i).fm.p(3)**2)
           if(outatint) then
              pt = pt*1000.
              pti = 1./pt
              call cyeta(w(i), y, eta)
              call cbst1(1, tg,  w(i), po)
              write(*,*) 
              write(*,'(5g14.4,i3)')
     *             sngl(eta), sngl(y),sngl(pt),sngl(pti),
     *             sngl(po.fm.p(4)/plab.fm.p(4)), w(i).code
           else
              aTrack.p = w(i)
              aTrack.pos.x = xin
              aTrack.pos.y = yin
              aTrack.pos.z = zin
              aTrack.t = i      ! instead of time, ptcl number is put
              aTrack.w.x = w(i).fm.p(1)/p               
              aTrack.w.y = w(i).fm.p(2)/p               
              aTrack.w.z = w(i).fm.p(3)/p               
              aTrack.wgt = 1.
              aTrack.cn = 0     ! comp. number unknown
              call epputTrack(aTrack)
           endif
        enddo
c              
        icon = 0
        end
        subroutine uafi1ev
c          this is called when all the initialization for
c         an event has been ended.  (incident particle has
c         set).  Do your own final init. for the event
        end

c       ***********************************************
c       *   This is called 
c       *     when a charged particle loses
c       *     energy in a component for which you have specifed to do
c       *     energy loss counting.
c       * Or
c       *     when a photon energy becomes lower than a given minimum
c       *     energy.  It's your decision whether you discard such 
c       *     a photon or count as energy lost here.
c  
        subroutine userde(info, aTrack, Move, media)
        implicit none
#include "ZepTrack.h"
#include "Zmove.h"
#include "Zmedia.h"

c          next is needed if you want to judge particle code.
c  #include "Zcode.h"     
         


        integer info             ! input. 0--> particle is still active.
                                 !        1--> particle is dying.
                                 !        (i.e, after moving Move.dl, it 
                                 !         dies) .
        record /epTrack/ aTrack  ! input. current track. before it is moved.
        record /epmove/  Move      ! input. containes info of moved track.
c                                ! output. You have to set Move.Abort
c         Move.Abort = 0 if you want to continue the simulation of
c                           this event normally.
c                    = 1 if you want to discard this particle
c                        For example, you may use this if you use
c                        formula for further development due to this
c                        particle 
c                    = 2 if you want to abort the generation of 
c                           this event, but want to execute ue1ev.
c                    = 3 if you want to abort the genration of 
c                         this event, and skip ue1ev.
        record /epmedia/media
c          You can use the following info.
c   ================================================================
c     aTrack.p         : the same particle record as Cosmos, i.e.,
c     aTrack.p.fm.p(1), p(2), p(3): momentum of the particle in GeV
c     aTrack.p.fm.p(4)            : total energy of    //
c     aTrack.p.mass               : mass  //                 
c     aTrack.p.code               : particle code 
c     aTrack.p.subcode            : particle subcode 
c     aTrack.p.charge             : charge  //.
c     aTrack.pos.x, y, z          : local coordinate of the particle in cm
c                                   If you need world coordinate, use
c                          call epl2w(aTrack.cn, aTrack.pos, wpos) 
c                    where
c                   record /epPos/ wpos is the output  world coordinate
c     aTrack.w.x, y, z            : direction cosines in the local coordinate
c                                   If you need world coordinate values, use
c                          call epl2wd(aTrack.cn, aTrack.w, ww)     
c                    where
c                   record /epDirec/ ww is the output  values in world coord.
c
c     aTrack.t                    : time in sum of (cm/beta) 
c     aTrack.wgt                  : weight of the particle when thinning is
c                                   done (normall 1.0)
c     aTrack.cn                   : component number
c
c     Move.Track      :   Info of moved track.
c     Move.boundary   :   Component boundary position when the particle crosses
c                         the boundary. (Move.Cross =T)
c                         Move.boundary.x, y, z. in cm. 
c     Move.proc       :   If the particle is going to interact, the process
c                            name is set. proc= one of
c                        'brem': bremstralhng
c                        'knoc': knock on (bhabha or moller scattering)
c                                other heavy particle knock-on
c                        'anih': positron anihilation
c                        'pair' : pair creation
c                        'comp' : compton scattering
c                        'phot' : photo electric effect
c                        'photop': photo-hadron production
c                        'coll'  : hadron's nuclear interaction
c                        'decay' : decay 
c                        '    '  : no interaction yet.
c    Move.dl          :  Path length in cm
c    Move.dE          :  Energy loss during dl. in GeV
c    Move.dEeff       :  Effective energy loss (use this for counting) GeV
c    Move.dEion       :  Energy loss due to ionization loss (GeV)
c    Move.dx          :  path length in g/cm^2
c    Move.dt          :  path length in r.l
c    Move.Cross       :  Becomes T if the ptcl crosses the boundary.
c    Move.Trunc       :  Becomes T if the ptcl track is truncated.
c    Move.Abort       :  must be set by the user.
c    
c    If you want to use a particle code, like,
c        if(aTrack.p.code .eq.  kphoton) ...
c    You have to uncomment Zcode.h above; they are the same as
c    Cosmos. Some of popular ones are:
c         kphoton:  photon             kelec  :  electron
c         kmuon  :  muon               kpion  :  pion
c         kkaon  :  kaon               knuc   :  nucleon
c         kneue  :  electron neutrino  kneumu :  muon neutrino
c         kalfa  :  He                 klibe  :  LiBeB group
c         kcno   :  CNO   group        khvy   :  Na/Mg/Si group
c         kvhvy  :  S/Cl/Ar group      kiron  :  Fe group
c         kdmes  :  D  meson           keta   :  eta  meson
c    Subcode may sometimes be needed:
c         regptcl:  particle           antip  : anti-particle
c         k0s    :  k0short            k0l    : k0 long
c 
        Move.Abort = 0
        end
c       ************************************************
c       *  This is called when a particle passes the boundary
c       *  of a component for which you have specifed to
c       *  count particle nubmer.
c
        subroutine userbd(info, aTrack, Move, media) 
        implicit none
#include "ZepTrack.h"
#include "Zmove.h"
#include "Zmedia.h"
#include "Zcode.h"     
c               use 1 for bondary trigger in config so that info
c               becomes > 0
        integer info              ! 0--> ptcl is exiting to void
                                  ! <0 --> ptcl is exiting to |info| comp.
                                  ! >0 --> ptcl is entering from info comp.
        record /epTrack/aTrack    ! input. Current track before it is moved.
                                  !  If info <=0, the track position is 
                                  !  somewhere inside the component from which
                                  !  the track is existing.
                                  !  If info >0, aTrack.pos is the position 
                                  !  just before exiting the prvious component
        record /epmove/ Move        ! input/output. 
                                  ! Move.Track is the track
                                  ! infomation of the current particle
                                  ! moved to a new position.
                                  ! Say, Move.Track.cn is the  current
                                  ! comp. number.  For other details,
                                  ! see userde.

c
c                      comp.1    comp.2
c          info<0    |  *-----x|          |        
c                                                   * is aTrack.pos
c                                                   x is Move.Track.pos
c          info>0    |        *|x         |       
c  
        record /epmedia/media       ! input.

        include 'Zother.h'
        record /ptcl/ w, po

        integer itag, cn, k
        real*8   yrap, eta, pt, ptinv, e, x, y, z
c         next one cannot be compliled so that 
c        use subroutine cpptcl
c        w = Move.Track.p
        call cpptcl(Move.Track.p, w)

        x = Move.Track.pos.x
        y = Move.Track.pos.y
        z = Move.Track.pos.z
        e = Move.Track.p.fm.p(4)
        cn = Move.Track.cn
        k = Move.Track.p.code

        if(.not. outatint) then
c                define output ptcls
c           if(k .eq. knuc .or. k .eq. kpion .or. k .eq. kkon) then
ccc           if(k .eq. kphoton) then
              pt= sqrt( w.fm.p(1)**2 + w.fm.p(2)**2)*1000. ! MeV
              ptinv = 1./pt
              call cyeta(w, yrap, eta)
c                    move to a target rest system
              call cbst1(1, tg, w, po)
              itag = Move.Track.t ! tag for particle number at production
              write(*, 
     *             '(f10.4, f10.4, 6g14.3,2i3,i5,2g14.4)')
     *             sngl(eta), sngl(yrap), sngl(pt),
     *             sngl(ptinv),
     *             sngl(e/energy), sngl(e), 
     *             sngl(po.fm.p(4)/plab.fm.p(4)),
     *             sngl(log10(e)), cn, k, itag,
     *             sngl(x), sngl(y)
ccc           endif
        endif


        Move.Abort = 0
        end
      subroutine cpptcl(a, b)
      implicit none
#include "Zptcl.h"
      record /ptcl/a, b
      b = a
      end

c       **************************************************
c       *  This is called when all the system level "end process"
c       *  for the event has been ended.  Do your own end process
c       *  for the event.
c       *
        subroutine ue1ev
        implicit none


        write(*,*) 0,0,0,0,0,0,0,0,0,0,0,0,0
        end
c       *************************************************
c       *  This is called when all the system level "end process"
c       * for all the events has been  ended. Do your own end
c       * process for the events

      subroutine ueaev
      implicit none


c      integer klena
c      character*8 uid
c      character*32 file

        call cerrorMsg(' all events simulated', 1)  ! error out        

        
c      call cgetLoginN(uid)
c      file = '/tmp/' // uid(1:klena(uid))// '/trace1'
c      write(*,*)'trace is in', file(1:klena(file))
      end
        subroutine  ephookPPCol(a, ntp)
c            set incident particles as those produced by p-p colliding  beam.
c     
          implicit none
#include  "Zptcl.h"
#include  "Zcode.h"
#include  "Zevhnv.h"
#include  "Zevhnp.h"
#include  "Zmanagerp.h"
          include 'Zother.h'
          

          record /ptcl/  a(*)
c             projectile and target information (both befor
c             and after collision ) in different system.
c
          integer icon, ntp

c

          do  while (.true.)
             call chncol(pj, tg, a, ntp, icon)
             if(icon .eq. 0) then
c                do j=1, ntp
c                   call cbst1(j, Cmsp,  a(j), a(j))
c                enddo
                goto 100
             endif
          enddo
 100      continue
        end
      subroutine outptcl(a, n, flag)
      implicit none
#include "Zptcl.h"
      integer n, flag
      record /ptcl/a(n)

      integer i

      do i = 1, n
         write(*,*) sngl(a(i).fm.p(1)),sngl(a(i).fm.p(2)),
     *    sngl(a(i).fm.p(3)), sngl(a(i).fm.p(4)), a(i).code, flag
      enddo
      end
      subroutine debugptcl(com, a)
      implicit none
#include "Zptcl.h"
      character*(*) com
      record /ptcl/ a
      write(*,*) com
      write(*,*) 'code=', a.code, a.fm.p(1), a.fm.p(2),
     *    a.fm.p(3), a.fm.p(4)
      end
